@@ -38,7 +38,10 @@ _cairo_boilerplate_test_fallback_create_surface (const char			 *name,
 						 cairo_content_t		  content,
 						 int				  width,
 						 int				  height,
+						 int				  max_width,
+						 int				  max_height,
 						 cairo_boilerplate_mode_t	  mode,
+						 int                              id,
 						 void				**closure)
 {
     *closure = NULL;
@@ -50,7 +53,10 @@ _cairo_boilerplate_test_meta_create_surface (const char			 *name,
 					     cairo_content_t		  content,
 					     int			  width,
 					     int			  height,
+					     int			  max_width,
+					     int			  max_height,
 					     cairo_boilerplate_mode_t	  mode,
+					     int                          id,
 					     void			**closure)
 {
     *closure = NULL;
@@ -72,31 +78,47 @@ _cairo_boilerplate_test_paginated_create_surface (const char			 *name,
 						  cairo_content_t		  content,
 						  int				  width,
 						  int				  height,
+						  int				  max_width,
+						  int				  max_height,
 						  cairo_boilerplate_mode_t	  mode,
+						  int                             id,
 						  void				**closure)
 {
     test_paginated_closure_t *tpc;
+    cairo_format_t format;
     cairo_surface_t *surface;
+    cairo_status_t status;
 
     *closure = tpc = xmalloc (sizeof (test_paginated_closure_t));
 
+    format = cairo_boilerplate_format_from_content (content);
+
     tpc->content = content;
     tpc->width = width;
     tpc->height = height;
-    tpc->stride = width * 4;
-
-    tpc->data = xcalloc (tpc->stride * height, 1);
+    tpc->stride = cairo_format_stride_for_width (format, width);
+    tpc->data = xcalloc (tpc->stride, height);
 
     surface = _cairo_test_paginated_surface_create_for_data (tpc->data,
-						       tpc->content,
-						       tpc->width,
-						       tpc->height,
-						       tpc->stride);
+							     tpc->content,
+							     tpc->width,
+							     tpc->height,
+							     tpc->stride);
+    if (cairo_surface_status (surface))
+	goto CLEANUP;
 
-    cairo_boilerplate_surface_set_user_data (surface,
-					     &test_paginated_closure_key,
-					     tpc, NULL);
+    status = cairo_surface_set_user_data (surface,
+					  &test_paginated_closure_key,
+					  tpc, NULL);
+    if (status == CAIRO_STATUS_SUCCESS)
+	return surface;
 
+    cairo_surface_destroy (surface);
+    surface = cairo_boilerplate_surface_create_in_error (status);
+
+  CLEANUP:
+    free (tpc->data);
+    free (tpc);
     return surface;
 }
 
@@ -137,16 +159,56 @@ _cairo_boilerplate_test_paginated_surface_write_to_png (cairo_surface_t	*surface
 						 tpc->stride);
 
     status = cairo_surface_write_to_png (image, filename);
-    if (status) {
-	CAIRO_BOILERPLATE_LOG ("Error writing %s: %s. Exiting\n",
-			       filename,
-			       cairo_status_to_string (status));
-	exit (1);
-    }
-
     cairo_surface_destroy (image);
 
-    return CAIRO_STATUS_SUCCESS;
+    return status;
+}
+
+cairo_surface_t *
+_cairo_boilerplate_test_paginated_get_image_surface (cairo_surface_t *surface,
+						     int page,
+						     int width,
+						     int height)
+{
+    cairo_format_t format;
+    test_paginated_closure_t *tpc;
+    cairo_status_t status;
+
+    /* XXX separate finish as per PDF */
+    if (page != 0)
+	return cairo_boilerplate_surface_create_in_error (CAIRO_STATUS_SURFACE_TYPE_MISMATCH);
+
+    /* show page first.  the automatic show_page is too late for us */
+    cairo_surface_show_page (surface);
+    status = cairo_surface_status (surface);
+    if (status)
+	return cairo_boilerplate_surface_create_in_error (status);
+
+    tpc = cairo_surface_get_user_data (surface, &test_paginated_closure_key);
+
+    format = cairo_boilerplate_format_from_content (tpc->content);
+
+    if (0) {
+	return cairo_image_surface_create_for_data (tpc->data + tpc->stride * (tpc->height - height) + 4 * (tpc->width - width), /* hide the device offset */
+						    format,
+						    width,
+						    height,
+						    tpc->stride);
+    } else {
+	cairo_surface_t *image, *surface;
+
+	image = cairo_image_surface_create_for_data (tpc->data,
+						     format,
+						     tpc->width,
+						     tpc->height,
+						     tpc->stride);
+	cairo_surface_set_device_offset (image,
+					 tpc->width - width,
+					 tpc->height - height);
+	surface = _cairo_boilerplate_get_image_surface (image, 0, width, height);
+	cairo_surface_destroy (image);
+	return surface;
+    }
 }
 
 void
