@@ -1,3 +1,4 @@
+/* -*- Mode: c; tab-width: 8; c-basic-offset: 4; indent-tabs-mode: t; -*- */
 /* cairo - a vector graphics library with display and print output
  *
  * Copyright © 2002 University of Southern California
@@ -53,15 +54,17 @@ _cairo_clip_init (cairo_clip_t *clip, cairo_surface_t *target)
     else
 	clip->mode = CAIRO_CLIP_MODE_MASK;
 
+    clip->all_clipped = FALSE;
+
     clip->surface = NULL;
     clip->surface_rect.x = 0;
     clip->surface_rect.y = 0;
-    clip->surface_rect.width = 0;
+    clip->surface_rect.width  = 0;
     clip->surface_rect.height = 0;
 
     clip->serial = 0;
 
-    pixman_region_init (&clip->region);
+    _cairo_region_init (&clip->region);
     clip->has_region = FALSE;
 
     clip->path = NULL;
@@ -72,19 +75,23 @@ _cairo_clip_init_copy (cairo_clip_t *clip, cairo_clip_t *other)
 {
     clip->mode = other->mode;
 
+    clip->all_clipped = other->all_clipped;
+
     clip->surface = cairo_surface_reference (other->surface);
     clip->surface_rect = other->surface_rect;
 
     clip->serial = other->serial;
 
-    pixman_region_init (&clip->region);
+    _cairo_region_init (&clip->region);
 
     if (other->has_region) {
-	if (pixman_region_copy (&clip->region, &other->region) !=
-		PIXMAN_REGION_STATUS_SUCCESS) {
-	    pixman_region_fini (&clip->region);
+	cairo_status_t status;
+
+	status = _cairo_region_copy (&clip->region, &other->region);
+	if (status) {
+	    _cairo_region_fini (&clip->region);
 	    cairo_surface_destroy (clip->surface);
-	    return CAIRO_STATUS_NO_MEMORY;
+	    return status;
 	}
         clip->has_region = TRUE;
     } else {
@@ -92,13 +99,15 @@ _cairo_clip_init_copy (cairo_clip_t *clip, cairo_clip_t *other)
     }
 
     clip->path = _cairo_clip_path_reference (other->path);
-    
+
     return CAIRO_STATUS_SUCCESS;
 }
 
 void
 _cairo_clip_reset (cairo_clip_t *clip)
 {
+    clip->all_clipped = FALSE;
+
     /* destroy any existing clip-region artifacts */
     cairo_surface_destroy (clip->surface);
     clip->surface = NULL;
@@ -106,11 +115,11 @@ _cairo_clip_reset (cairo_clip_t *clip)
     clip->serial = 0;
 
     if (clip->has_region) {
-        /* pixman_region_fini just releases the resources used but
+        /* _cairo_region_fini just releases the resources used but
          * doesn't bother with leaving the region in a valid state.
-         * So pixman_region_init has to be called afterwards. */
-	pixman_region_fini (&clip->region);
-        pixman_region_init (&clip->region);
+         * So _cairo_region_init has to be called afterwards. */
+	_cairo_region_fini (&clip->region);
+        _cairo_region_init (&clip->region);
 
         clip->has_region = FALSE;
     }
@@ -119,17 +128,30 @@ _cairo_clip_reset (cairo_clip_t *clip)
     clip->path = NULL;
 }
 
+static void
+_cairo_clip_set_all_clipped (cairo_clip_t *clip, cairo_surface_t *target)
+{
+    _cairo_clip_reset (clip);
+
+    clip->all_clipped = TRUE;
+    clip->serial = _cairo_surface_allocate_clip_serial (target);
+}
+
+
 static cairo_status_t
 _cairo_clip_path_intersect_to_rectangle (cairo_clip_path_t       *clip_path,
-   				         cairo_rectangle_int16_t *rectangle)
+				         cairo_rectangle_int_t   *rectangle)
 {
     while (clip_path) {
         cairo_status_t status;
         cairo_traps_t traps;
         cairo_box_t extents;
-        cairo_rectangle_int16_t extents_rect;
+        cairo_rectangle_int_t extents_rect;
+
+	_cairo_box_from_rectangle (&extents, rectangle);
 
         _cairo_traps_init (&traps);
+	_cairo_traps_limit (&traps, &extents);
 
         status = _cairo_path_fixed_fill_to_traps (&clip_path->path,
                                                   clip_path->fill_rule,
@@ -141,11 +163,12 @@ _cairo_clip_path_intersect_to_rectangle (cairo_clip_path_t       *clip_path,
         }
 
         _cairo_traps_extents (&traps, &extents);
-        _cairo_box_round_to_rectangle (&extents, &extents_rect);
-        _cairo_rectangle_intersect (rectangle, &extents_rect);
-
         _cairo_traps_fini (&traps);
 
+        _cairo_box_round_to_rectangle (&extents, &extents_rect);
+        if (! _cairo_rectangle_intersect (rectangle, &extents_rect))
+	    return CAIRO_STATUS_SUCCESS;
+
         clip_path = clip_path->prev;
     }
 
@@ -154,14 +177,20 @@ _cairo_clip_path_intersect_to_rectangle (cairo_clip_path_t       *clip_path,
 
 cairo_status_t
 _cairo_clip_intersect_to_rectangle (cairo_clip_t            *clip,
-				    cairo_rectangle_int16_t *rectangle)
+				    cairo_rectangle_int_t *rectangle)
 {
+    cairo_status_t status;
+    cairo_bool_t is_empty;
+
     if (!clip)
 	return CAIRO_STATUS_SUCCESS;
 
+    if (clip->all_clipped) {
+	*rectangle = clip->surface_rect;
+	return CAIRO_STATUS_SUCCESS;
+    }
+
     if (clip->path) {
-        cairo_status_t status;
-        
         status = _cairo_clip_path_intersect_to_rectangle (clip->path,
                                                           rectangle);
         if (status)
@@ -169,65 +198,67 @@ _cairo_clip_intersect_to_rectangle (cairo_clip_t            *clip,
     }
 
     if (clip->has_region) {
-	cairo_status_t status = CAIRO_STATUS_SUCCESS;
-	pixman_region16_t intersection;
+	cairo_region_t intersection;
 
-	pixman_region_init_rect (&intersection,
-                                  rectangle->x, rectangle->y,
-                                  rectangle->width, rectangle->height);
+	_cairo_region_init_rect (&intersection, rectangle);
 
-	if (PIXMAN_REGION_STATUS_SUCCESS !=
-            pixman_region_intersect (&intersection, &clip->region,
-                                     &intersection)) {
-	    status = CAIRO_STATUS_NO_MEMORY;
-	} else {
-            _cairo_region_extents_rectangle (&intersection, rectangle);
-        }
+	status = _cairo_region_intersect (&intersection, &clip->region,
+					  &intersection);
 
-        pixman_region_fini (&intersection);
+	if (!status)
+	    _cairo_region_get_extents (&intersection, rectangle);
+
+        _cairo_region_fini (&intersection);
 
         if (status)
             return status;
     }
 
     if (clip->surface)
-	_cairo_rectangle_intersect (rectangle, &clip->surface_rect);
+	is_empty = _cairo_rectangle_intersect (rectangle, &clip->surface_rect);
 
     return CAIRO_STATUS_SUCCESS;
 }
 
 cairo_status_t
 _cairo_clip_intersect_to_region (cairo_clip_t      *clip,
-				 pixman_region16_t *region)
+				 cairo_region_t *region)
 {
-    pixman_region_status_t pixman_status;
+    cairo_status_t status;
 
     if (!clip)
 	return CAIRO_STATUS_SUCCESS;
 
+    if (clip->all_clipped) {
+	cairo_region_t clip_rect;
+
+	_cairo_region_init_rect (&clip_rect, &clip->surface_rect);
+
+	status = _cairo_region_intersect (region, &clip_rect, region);
+
+	_cairo_region_fini (&clip_rect);
+
+	return status;
+    }
+
     if (clip->path) {
 	/* Intersect clip path into region. */
     }
 
     if (clip->has_region) {
-	pixman_status = pixman_region_intersect (region, &clip->region, region);
-	if (pixman_status != PIXMAN_REGION_STATUS_SUCCESS)
-	    return CAIRO_STATUS_NO_MEMORY;
+	status = _cairo_region_intersect (region, &clip->region, region);
+	if (status)
+	    return status;
     }
 
     if (clip->surface) {
-	cairo_status_t status = CAIRO_STATUS_SUCCESS;
-	pixman_region16_t clip_rect;
+	cairo_region_t clip_rect;
 
-        pixman_region_init_rect (&clip_rect,
-                                  clip->surface_rect.x, clip->surface_rect.y,
-                                  clip->surface_rect.width, clip->surface_rect.height);
+	_cairo_region_init_rect (&clip_rect, &clip->surface_rect);
 
-        if (PIXMAN_REGION_STATUS_SUCCESS !=
-            pixman_region_intersect (region, &clip_rect, region))
-	    status = CAIRO_STATUS_NO_MEMORY;
+	status = _cairo_region_intersect (region, &clip_rect, region);
 
-        pixman_region_fini (&clip_rect);
+	_cairo_region_fini (&clip_rect);
 
         if (status)
             return status;
@@ -246,12 +277,15 @@ _cairo_clip_combine_to_surface (cairo_clip_t                  *clip,
 				cairo_surface_t               *dst,
 				int                           dst_x,
 				int                           dst_y,
-				const cairo_rectangle_int16_t *extents)
+				const cairo_rectangle_int_t *extents)
 {
-    cairo_pattern_union_t pattern;
+    cairo_surface_pattern_t pattern;
     cairo_status_t status;
 
-    _cairo_pattern_init_for_surface (&pattern.surface, clip->surface);
+    if (clip->all_clipped)
+	return CAIRO_STATUS_SUCCESS;
+
+    _cairo_pattern_init_for_surface (&pattern, clip->surface);
 
     status = _cairo_surface_composite (op,
 				       &pattern.base,
@@ -284,7 +318,7 @@ _cairo_clip_intersect_path (cairo_clip_t       *clip,
 
     clip_path = malloc (sizeof (cairo_clip_path_t));
     if (clip_path == NULL)
-	return CAIRO_STATUS_NO_MEMORY;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
     status = _cairo_path_fixed_init_copy (&clip_path->path, path);
     if (status) {
@@ -292,7 +326,7 @@ _cairo_clip_intersect_path (cairo_clip_t       *clip,
 	return status;
     }
 
-    clip_path->ref_count = 1;
+    CAIRO_REFERENCE_COUNT_INIT (&clip_path->ref_count, 1);
     clip_path->fill_rule = fill_rule;
     clip_path->tolerance = tolerance;
     clip_path->antialias = antialias;
@@ -308,7 +342,9 @@ _cairo_clip_path_reference (cairo_clip_path_t *clip_path)
     if (clip_path == NULL)
 	return NULL;
 
-    clip_path->ref_count++;
+    assert (CAIRO_REFERENCE_COUNT_HAS_REFERENCE (&clip_path->ref_count));
+
+    _cairo_reference_count_inc (&clip_path->ref_count);
 
     return clip_path;
 }
@@ -319,8 +355,9 @@ _cairo_clip_path_destroy (cairo_clip_path_t *clip_path)
     if (clip_path == NULL)
 	return;
 
-    clip_path->ref_count--;
-    if (clip_path->ref_count)
+    assert (CAIRO_REFERENCE_COUNT_HAS_REFERENCE (&clip_path->ref_count));
+
+    if (! _cairo_reference_count_dec_and_test (&clip_path->ref_count))
 	return;
 
     _cairo_path_fixed_fini (&clip_path->path);
@@ -328,14 +365,18 @@ _cairo_clip_path_destroy (cairo_clip_path_t *clip_path)
     free (clip_path);
 }
 
+
 static cairo_int_status_t
 _cairo_clip_intersect_region (cairo_clip_t    *clip,
 			      cairo_traps_t   *traps,
 			      cairo_surface_t *target)
 {
-    pixman_region16_t region;
+    cairo_region_t region;
     cairo_int_status_t status;
 
+    if (clip->all_clipped)
+	return CAIRO_STATUS_SUCCESS;
+
     if (clip->mode != CAIRO_CLIP_MODE_REGION)
 	return CAIRO_INT_STATUS_UNSUPPORTED;
 
@@ -344,31 +385,30 @@ _cairo_clip_intersect_region (cairo_clip_t    *clip,
     if (status)
 	return status;
 
-    status = CAIRO_STATUS_SUCCESS;
-
     if (!clip->has_region) {
-        if (pixman_region_copy (&clip->region, &region) ==
-		PIXMAN_REGION_STATUS_SUCCESS)
+        status = _cairo_region_copy (&clip->region, &region);
+	if (status == CAIRO_STATUS_SUCCESS)
 	    clip->has_region = TRUE;
-	else
-	    status = CAIRO_STATUS_NO_MEMORY;
     } else {
-	pixman_region16_t intersection;
-        pixman_region_init (&intersection);
+	cairo_region_t intersection;
 
-	if (PIXMAN_REGION_STATUS_SUCCESS !=
-		pixman_region_intersect (&intersection,
+        _cairo_region_init (&intersection);
+
+	status = _cairo_region_intersect (&intersection,
 		                         &clip->region,
-		                         &region) ||
-	    PIXMAN_REGION_STATUS_SUCCESS !=
-		pixman_region_copy (&clip->region, &intersection))
-	    status = CAIRO_STATUS_NO_MEMORY;
+		                         &region);
 
-        pixman_region_fini (&intersection);
+	if (status == CAIRO_STATUS_SUCCESS)
+	    status = _cairo_region_copy (&clip->region, &intersection);
+
+        _cairo_region_fini (&intersection);
     }
 
     clip->serial = _cairo_surface_allocate_clip_serial (target);
-    pixman_region_fini (&region);
+    _cairo_region_fini (&region);
+
+    if (! _cairo_region_not_empty (&clip->region))
+	_cairo_clip_set_all_clipped (clip, target);
 
     return status;
 }
@@ -381,10 +421,13 @@ _cairo_clip_intersect_mask (cairo_clip_t      *clip,
 {
     cairo_pattern_union_t pattern;
     cairo_box_t extents;
-    cairo_rectangle_int16_t surface_rect, target_rect;
-    cairo_surface_t *surface;
+    cairo_rectangle_int_t surface_rect, target_rect;
+    cairo_surface_t *surface = NULL;
     cairo_status_t status;
 
+    if (clip->all_clipped)
+	return CAIRO_STATUS_SUCCESS;
+
     /* Represent the clip as a mask surface.  We create a new surface
      * the size of the intersection of the old mask surface and the
      * extents of the new clip path. */
@@ -392,32 +435,69 @@ _cairo_clip_intersect_mask (cairo_clip_t      *clip,
     _cairo_traps_extents (traps, &extents);
     _cairo_box_round_to_rectangle (&extents, &surface_rect);
 
-    if (clip->surface != NULL)
-	_cairo_rectangle_intersect (&surface_rect, &clip->surface_rect);
+    if (clip->surface != NULL) {
+	if (! _cairo_rectangle_intersect (&surface_rect, &clip->surface_rect))
+	    goto DONE;
+    }
 
     /* Intersect with the target surface rectangle so we don't use
      * more memory and time than we need to. */
-
     status = _cairo_surface_get_extents (target, &target_rect);
-    if (!status)
-	_cairo_rectangle_intersect (&surface_rect, &target_rect);
+    if (status == CAIRO_STATUS_SUCCESS) {
+	if (! _cairo_rectangle_intersect (&surface_rect, &target_rect))
+	    goto DONE;
+    }
+
+    if (surface_rect.width == 0 || surface_rect.height == 0)
+	goto DONE;
+
+    _cairo_pattern_init_solid (&pattern.solid, CAIRO_COLOR_WHITE,
+			       CAIRO_CONTENT_COLOR);
+    /* The clipping operation should ideally be something like the following to
+     * avoid having to do as many passes over the data
+
+	if (clip->surface != NULL) {
+	    _cairo_pattern_init_for_surface (&pattern.surface, clip->surface);
+	} else {
+	    _cairo_pattern_init_solid (&pattern.solid, CAIRO_COLOR_WHITE,
+			       CAIRO_CONTENT_COLOR);
+	}
+	status = _cairo_surface_composite_trapezoids (CAIRO_OPERATOR_IN,
+						  &pattern.base,
+						  surface,
+						  antialias,
+						  0, 0,
+						  0, 0,
+						  surface_rect.width,
+						  surface_rect.height,
+						  traps->traps,
+						  traps->num_traps);
+
+	However this operation is not accelerated by pixman
+
+	I believe the best possible operation would probably an unbounded SRC
+	operator.  Using SRC we could potentially avoid having to initialize
+	the surface which would be ideal from an efficiency point of view.
+	However, _cairo_surface_composite_trapezoids (CAIRO_OPERATOR_SOURCE) is
+	bounded by the mask.
+
+    */
 
     surface = _cairo_surface_create_similar_solid (target,
 						   CAIRO_CONTENT_ALPHA,
 						   surface_rect.width,
 						   surface_rect.height,
-						   CAIRO_COLOR_WHITE,
-						   NULL);
-    if (surface->status)
-	return CAIRO_STATUS_NO_MEMORY;
+						   CAIRO_COLOR_TRANSPARENT);
+    if (surface->status) {
+	_cairo_pattern_fini (&pattern.base);
+	return surface->status;
+    }
 
     /* Render the new clipping path into the new mask surface. */
 
     _cairo_traps_translate (traps, -surface_rect.x, -surface_rect.y);
-    _cairo_pattern_init_solid (&pattern.solid, CAIRO_COLOR_WHITE,
-			       CAIRO_CONTENT_COLOR);
 
-    status = _cairo_surface_composite_trapezoids (CAIRO_OPERATOR_IN,
+    status = _cairo_surface_composite_trapezoids (CAIRO_OPERATOR_ADD,
 						  &pattern.base,
 						  surface,
 						  antialias,
@@ -459,14 +539,17 @@ _cairo_clip_intersect_mask (cairo_clip_t      *clip,
 	    cairo_surface_destroy (surface);
 	    return status;
 	}
-
-	cairo_surface_destroy (clip->surface);
     }
 
+ DONE:
+    cairo_surface_destroy (clip->surface);
     clip->surface = surface;
     clip->surface_rect = surface_rect;
     clip->serial = _cairo_surface_allocate_clip_serial (target);
 
+    if (surface_rect.width == 0 || surface_rect.height == 0)
+	_cairo_clip_set_all_clipped (clip, target);
+
     return status;
 }
 
@@ -479,8 +562,18 @@ _cairo_clip_clip (cairo_clip_t       *clip,
 		  cairo_surface_t    *target)
 {
     cairo_status_t status;
+    cairo_rectangle_int_t rectangle;
     cairo_traps_t traps;
 
+    if (clip->all_clipped)
+	return CAIRO_STATUS_SUCCESS;
+
+    /* catch the empty clip path */
+    if (! path->has_current_point) {
+	_cairo_clip_set_all_clipped (clip, target);
+	return CAIRO_STATUS_SUCCESS;
+    }
+
     status = _cairo_clip_intersect_path (clip,
 					 path, fill_rule, tolerance,
 					 antialias);
@@ -491,6 +584,17 @@ _cairo_clip_clip (cairo_clip_t       *clip,
 	return status;
 
     _cairo_traps_init (&traps);
+
+    /* Limit the traps to the target surface
+     * - so we don't add more traps than needed. */
+    status = _cairo_surface_get_extents (target, &rectangle);
+    if (status == CAIRO_STATUS_SUCCESS) {
+	cairo_box_t box;
+
+	_cairo_box_from_rectangle (&box, &rectangle);
+	_cairo_traps_limit (&traps, &box);
+    }
+
     status = _cairo_path_fixed_fill_to_traps (path,
 					      fill_rule,
 					      tolerance,
@@ -515,8 +619,11 @@ _cairo_clip_translate (cairo_clip_t  *clip,
                        cairo_fixed_t  tx,
                        cairo_fixed_t  ty)
 {
+    if (clip->all_clipped)
+	return;
+
     if (clip->has_region) {
-        pixman_region_translate (&clip->region,
+        _cairo_region_translate (&clip->region,
                                  _cairo_fixed_integer_part (tx),
                                  _cairo_fixed_integer_part (ty));
     }
@@ -535,24 +642,29 @@ _cairo_clip_translate (cairo_clip_t  *clip,
 				     _cairo_fixed_to_double (ty));
 
         while (clip_path) {
-            _cairo_path_fixed_device_transform (&clip_path->path, &matrix);
+            _cairo_path_fixed_transform (&clip_path->path, &matrix);
             clip_path = clip_path->prev;
         }
     }
 }
 
-static void
+static cairo_status_t
 _cairo_clip_path_reapply_clip_path (cairo_clip_t      *clip,
                                     cairo_clip_path_t *clip_path)
 {
-    if (clip_path->prev)
-        _cairo_clip_path_reapply_clip_path (clip, clip_path->prev);
+    cairo_status_t status;
 
-    _cairo_clip_intersect_path (clip,
-                                &clip_path->path,
-                                clip_path->fill_rule,
-                                clip_path->tolerance,
-                                clip_path->antialias);
+    if (clip_path->prev) {
+        status = _cairo_clip_path_reapply_clip_path (clip, clip_path->prev);
+	if (status && status != CAIRO_INT_STATUS_UNSUPPORTED)
+	    return status;
+    }
+
+    return _cairo_clip_intersect_path (clip,
+                                       &clip_path->path,
+				       clip_path->fill_rule,
+				       clip_path->tolerance,
+				       clip_path->antialias);
 }
 
 cairo_status_t
@@ -560,6 +672,8 @@ _cairo_clip_init_deep_copy (cairo_clip_t    *clip,
                             cairo_clip_t    *other,
                             cairo_surface_t *target)
 {
+    cairo_status_t status;
+
     _cairo_clip_init (clip, target);
 
     if (other->mode != clip->mode) {
@@ -567,26 +681,36 @@ _cairo_clip_init_deep_copy (cairo_clip_t    *clip,
          * whatever the right handling is happen */
     } else {
         if (other->has_region) {
-            if (pixman_region_copy (&clip->region, &other->region) !=
-		    PIXMAN_REGION_STATUS_SUCCESS)
+            status = _cairo_region_copy (&clip->region, &other->region);
+	    if (status)
 		goto BAIL;
+
 	    clip->has_region = TRUE;
         }
 
         if (other->surface) {
-            if (_cairo_surface_clone_similar (target, other->surface,
-					  other->surface_rect.x,
-					  other->surface_rect.y,
-					  other->surface_rect.width,
-					  other->surface_rect.height,
-					  &clip->surface) !=
-		    CAIRO_STATUS_SUCCESS)
+	    int dx, dy;
+            status = _cairo_surface_clone_similar (target, other->surface,
+					           0,
+						   0,
+						   other->surface_rect.width,
+						   other->surface_rect.height,
+						   &dx, &dy,
+						   &clip->surface);
+	    if (status)
 		goto BAIL;
+
             clip->surface_rect = other->surface_rect;
+
+	    /* src offset was 0, so we expect an exact replica of the surface */
+	    assert (dx == 0);
+	    assert (dy == 0);
         }
 
         if (other->path) {
-            _cairo_clip_path_reapply_clip_path (clip, other->path);
+            status = _cairo_clip_path_reapply_clip_path (clip, other->path);
+	    if (status && status != CAIRO_INT_STATUS_UNSUPPORTED)
+		goto BAIL;
         }
     }
 
@@ -594,11 +718,11 @@ _cairo_clip_init_deep_copy (cairo_clip_t    *clip,
 
 BAIL:
     if (clip->has_region)
-	pixman_region_fini (&clip->region);
+	_cairo_region_fini (&clip->region);
     if (clip->surface)
 	cairo_surface_destroy (clip->surface);
 
-    return CAIRO_STATUS_NO_MEMORY;
+    return status;
 }
 
 const cairo_rectangle_list_t _cairo_rectangles_nil =
@@ -607,78 +731,105 @@ static const cairo_rectangle_list_t _cairo_rectangles_not_representable =
   { CAIRO_STATUS_CLIP_NOT_REPRESENTABLE, NULL, 0 };
 
 static cairo_bool_t
-_cairo_clip_rect_to_user (cairo_gstate_t *gstate,
-                          double x, double y, double width, double height,
-                          cairo_rectangle_t *rectangle)
+_cairo_clip_int_rect_to_user (cairo_gstate_t *gstate,
+			      cairo_rectangle_int_t *clip_rect,
+			      cairo_rectangle_t *user_rect)
 {
-    double x2 = x + width;
-    double y2 = y + height;
     cairo_bool_t is_tight;
 
-    _cairo_gstate_backend_to_user_rectangle (gstate, &x, &y, &x2, &y2, &is_tight);
-    rectangle->x = x;
-    rectangle->y = y;
-    rectangle->width = x2 - x;
-    rectangle->height = y2 - y;
+    double x1 = clip_rect->x;
+    double y1 = clip_rect->y;
+    double x2 = clip_rect->x + (int) clip_rect->width;
+    double y2 = clip_rect->y + (int) clip_rect->height;
+
+    _cairo_gstate_backend_to_user_rectangle (gstate,
+					     &x1, &y1, &x2, &y2,
+					     &is_tight);
+
+    user_rect->x = x1;
+    user_rect->y = y1;
+    user_rect->width  = x2 - x1;
+    user_rect->height = y2 - y1;
+
     return is_tight;
 }
 
-cairo_private cairo_rectangle_list_t*
+cairo_rectangle_list_t *
 _cairo_clip_copy_rectangle_list (cairo_clip_t *clip, cairo_gstate_t *gstate)
 {
     cairo_rectangle_list_t *list;
-    cairo_rectangle_t *rectangles;
-    int n_boxes;
+    cairo_rectangle_t *rectangles = NULL;
+    int n_boxes = 0;
 
-    if (clip->path || clip->surface)
-        return (cairo_rectangle_list_t*) &_cairo_rectangles_not_representable;
+    if (clip->all_clipped)
+	goto DONE;
 
-    n_boxes = clip->has_region ? pixman_region_num_rects (&clip->region) : 1;
-    if (n_boxes > 0) {
-	rectangles = _cairo_malloc_ab (n_boxes, sizeof (cairo_rectangle_t));
-	if (rectangles == NULL)
-	    return (cairo_rectangle_list_t*) &_cairo_rectangles_nil;
-    } else {
-	rectangles = NULL;
+    if (clip->path || clip->surface) {
+	_cairo_error_throw (CAIRO_STATUS_CLIP_NOT_REPRESENTABLE);
+	return (cairo_rectangle_list_t*) &_cairo_rectangles_not_representable;
     }
 
     if (clip->has_region) {
-        pixman_box16_t *boxes;
+	cairo_box_int_t *boxes;
         int i;
-        
-        boxes = pixman_region_rects (&clip->region);
-        for (i = 0; i < n_boxes; ++i) {
-            if (!_cairo_clip_rect_to_user(gstate, boxes[i].x1, boxes[i].y1,
-                                          boxes[i].x2 - boxes[i].x1,
-                                          boxes[i].y2 - boxes[i].y1,
-                                          &rectangles[i])) {
-                free (rectangles);
-                return (cairo_rectangle_list_t*)
-                    &_cairo_rectangles_not_representable;
-            }
-        }
+
+	if (_cairo_region_get_boxes (&clip->region, &n_boxes, &boxes))
+	    return (cairo_rectangle_list_t*) &_cairo_rectangles_nil;
+
+	if (n_boxes) {
+	    rectangles = _cairo_malloc_ab (n_boxes, sizeof (cairo_rectangle_t));
+	    if (rectangles == NULL) {
+		_cairo_region_boxes_fini (&clip->region, boxes);
+		_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
+		return (cairo_rectangle_list_t*) &_cairo_rectangles_nil;
+	    }
+
+	    for (i = 0; i < n_boxes; ++i) {
+               cairo_rectangle_int_t clip_rect;
+
+               clip_rect.x = boxes[i].p1.x;
+               clip_rect.y = boxes[i].p1.y;
+               clip_rect.width  = boxes[i].p2.x - boxes[i].p1.x;
+               clip_rect.height = boxes[i].p2.y - boxes[i].p1.y;
+
+		if (!_cairo_clip_int_rect_to_user(gstate, &clip_rect, &rectangles[i])) {
+		    _cairo_error_throw (CAIRO_STATUS_CLIP_NOT_REPRESENTABLE);
+		    _cairo_region_boxes_fini (&clip->region, boxes);
+		    free (rectangles);
+		    return (cairo_rectangle_list_t*) &_cairo_rectangles_not_representable;
+		}
+	    }
+	}
+
+	_cairo_region_boxes_fini (&clip->region, boxes);
     } else {
-        cairo_rectangle_int16_t extents;
-        if (_cairo_surface_get_extents (_cairo_gstate_get_target (gstate),
-		                        &extents) != CAIRO_STATUS_SUCCESS) {
-            free (rectangles);
+        cairo_rectangle_int_t extents;
+
+	n_boxes = 1;
+
+	rectangles = malloc(sizeof (cairo_rectangle_t));
+	if (rectangles == NULL) {
+	    _cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
 	    return (cairo_rectangle_list_t*) &_cairo_rectangles_nil;
 	}
-	if (! _cairo_clip_rect_to_user(gstate, extents.x, extents.y,
-                                          extents.width, extents.height,
-                                          rectangles)) {
-            free (rectangles);
-            return (cairo_rectangle_list_t*)
-                &_cairo_rectangles_not_representable;
-        }
+
+	if (_cairo_surface_get_extents (_cairo_gstate_get_target (gstate), &extents) ||
+	    !_cairo_clip_int_rect_to_user(gstate, &extents, rectangles))
+	{
+	    _cairo_error_throw (CAIRO_STATUS_CLIP_NOT_REPRESENTABLE);
+	    free (rectangles);
+	    return (cairo_rectangle_list_t*) &_cairo_rectangles_not_representable;
+	}
     }
 
+ DONE:
     list = malloc (sizeof (cairo_rectangle_list_t));
     if (list == NULL) {
+	_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
         free (rectangles);
         return (cairo_rectangle_list_t*) &_cairo_rectangles_nil;
     }
-    
+
     list->status = CAIRO_STATUS_SUCCESS;
     list->rectangles = rectangles;
     list->num_rectangles = n_boxes;
