@@ -30,10 +30,9 @@
 
 static cairo_test_draw_function_t draw;
 
-cairo_test_t test = {
+static const cairo_test_t test = {
     "text-pattern",
-    "Patterned Text\n"
-    "PDF backend will not be tested due to limitations in poppler.",
+    "Patterned Text\n",
     IMAGE_WIDTH, IMAGE_HEIGHT,
     draw
 };
